

from neurospeed.utils.helper_service import UtilService as utils
from pathlib import Path
import os
import logging
from constants import PROD_API_CONFIG

class ApiConfig:

    def __init__(self, api_config = PROD_API_CONFIG):
        self._contex = "ApiConfig - "
        
        '''
        precedence of config path options: 
        
        if C:/neurobrave/hia_config.json exists, it's used
        otherwise, if /config/api_config.json exists under the runtime folder, its' used
        it none of these exist, defauilt that comes with the neurospeed library is used.
            
        '''
        
        self._api_config = api_config
        self._api_http_url = api_config["api_address"]
        self._pipeline_url = api_config["pipeline_address"]
        
        # print("api config path: " + self._config_path)
        # print("{} NeuroSpeed API HTTP URL: {} ".format(self._contex, self._api_http_url))
        # print("{} NeuroSpeed API SOCKET URL: {} ".format(self._contex, self._pipeline_url))
        
        logging.debug("{} NeuroSpeed API HTTP URL: {} ".format(self._contex, self._api_http_url))
        logging.debug("{} NeuroSpeed API SOCKET URL: {} ".format(self._contex, self._pipeline_url))
        
    def get_http_url(self):
        return self._api_http_url
    
    def get_socket_url(self):
        return self._pipeline_url
        


# -*- coding: utf-8 -*-
"""
Created on Sat Sep 11 20:12:14 2021

@author: NeuroBrave
"""

from neurospeed.utils.http_service import HttpService
import copy 
import json
from constants import PROD_API_CONFIG

class Gateway_HttpHandler:
    
    def __init__(self, customer_auth_handler, api_config =PROD_API_CONFIG):
        self._customer_auth  = customer_auth_handler
        self._http_service = HttpService(api_config=api_config)
        
        if customer_auth_handler.get_access_token() is None: 
            raise ValueError("no token for customer, failed to initialize gateway")
        self._headers =  {
                "Authorization": "Bearer " + customer_auth_handler.get_access_token()
        }

    # returns list of connected users : 
    # [{id: number; username: string;}, ..., {id: number; username: string;}]
    def get_connected_users(self):
        endpoint = "/gateway/users/connected"
        params = {}
        response = self._http_service.GET_request(endpoint, params, self._headers)
        return response

    def get_connections(self, page_number = 1, page_size = 50, filters={}): 
        endpoint = "/gateway/connections"

        payload  = {
            "pager": {
                "page_number": page_number,
                "page_size":page_size
            },
            "filters": filters
        }
        headers_copy = copy.deepcopy(self._headers)
        headers_copy["Content-type"] = "application/json"
        response = self._http_service.POST_request(endpoint, payload, headers_copy)
        return response
     
        



# -*- coding: utf-8 -*-
"""
Created on Fri Sep 11 20:07:52 2021

@author: NeuroBrave
"""


import requests
import json
from neurospeed.utils.api_config_service import ApiConfig
import copy 
import logging
from constants import PROD_API_CONFIG

class HttpService:    
    
    def __init__(self, api_config = PROD_API_CONFIG):
        api_config = ApiConfig(api_config)

        self._api_url =  api_config.get_http_url()
        self._api_url = self._api_url + '/api'
    
    
    
    def GET_request(self, endpoint, params, headers = {}):
        api_url = self._api_url + endpoint
        logging.debug(f"HTTP GET to {api_url} with params: {params}")
        
        response = None
        try:
            response = requests.get(api_url, headers=headers, params = params)
            response.raise_for_status()
      
            if (response.ok):
                response_parsed = json.loads(response.content)
                logging.debug(f"GET response from {endpoint}")
                return response_parsed
    
            else:
                raise requests.exceptions.HTTPError
        
        except requests.exceptions.HTTPError as e:
            print('Failed to HTTP GET to url:', api_url)
            print (e.response.text)
            
            
    def POST_request(self, endpoint, payload, headers = {}):
        api_url = self._api_url + endpoint
        logging.debug(f"HTTP POST to {api_url} with params: {payload}")
        try:
            clean_payload = copy.deepcopy(payload)
            for key in payload:
                if key == "password":
                    clean_payload.pop(key)
                    
            logging.debug(f"doing HTTP POST to {api_url} payload {clean_payload}")     
        except Exception as err:
            pass
           
        response = None 
        try:
            response = requests.post(api_url, json=payload, headers =headers)
            response.raise_for_status()
    
     
            if (response.ok):
                # Handle empty response content
                if response.content.strip() == b'':
                    logging.debug(f"POST response from {endpoint} is empty.")
                    return None  # or return {} if you expect a dictionary
                
                response_parsed = json.loads(response.content)
                logging.debug("POST response from {endpoint}:{response_parsed}")
                return response_parsed
    
            else:
                raise requests.exceptions.HTTPError
        
        except requests.exceptions.HTTPError as e:
            print('Failed to HTTP POST to url:', api_url)
            print (e.response.text)      
            logging.debug(f"Failed to HTTP POST to {api_url}: {e.response.text}")
            
    def PUT_request(self, endpoint, payload, params, headers = {}):
        api_url = self._api_url + endpoint
        print('doing HTTP PUT to url:', api_url, 'with params:', params, 'and payload:', payload, '')
        
        response = None
        try:
            response = requests.put(api_url, data=payload, headers =headers, params=params)
            response.raise_for_status()
            if (response.ok):
                response_parsed = json.loads(response.content)
                print('PUT response from {} '.format(endpoint ))
                return response_parsed
    
            else:
                raise requests.exceptions.HTTPError
        
        except requests.exceptions.HTTPError as e:
            print('Failed to HTTP PUT to url:', api_url)
            print (e.response.text)      
            logging.debug(f"Failed to HTTP PUT to {api_url}: {e.response.text}")

    def DELETE_request(self, endpoint, params, headers = {}):
        api_url = self._api_url + endpoint
        print('doing HTTP DELETE to url:', api_url, 'with params:', params)
        
        response = None
        try:
            response = requests.delete(api_url, headers =headers, params=params)
            response.raise_for_status()

            if (response.ok):
                response_parsed = json.loads(response.content)
                print('DELETE response from {} '.format(endpoint ))
                return response_parsed
    
            else:
                raise requests.exceptions.HTTPError
        
        except requests.exceptions.HTTPError as e:
            print('Failed to HTTP DELETE to url:', api_url)
            print (e.response.text)       
            logging.debug(f"Failed to HTTP DELETE to {api_url}: {e.response.text}")
  
   


# -*- coding: utf-8 -*-
"""
Created on Tue Sep 14 13:45:53 2021

@author: NeuroBrave
"""

from neurospeed.utils.http_service import HttpService
import copy 
import json
from constants import PROD_API_CONFIG

class UserRoom_Recorder_Handler:
    
    def __init__(self, customer_auth_handler, api_config = PROD_API_CONFIG):
        # create recorder http api instance
        self._recorder_api = self.Recorder_HttpApi(customer_auth_handler, api_config=api_config)
        
    # there can only be one recorder at a time per USER.
    # previous active recorder must be stopped in order to create another one
    # data still can be exported from stopped recorder, but recorder itself cannot be activated again
    def create_recorder(self, username, recorder_name = None):
        create_recorder = {
            "username": username,
            "recorder_name": recorder_name, 
            "unlimited": "true"
        }
        response = self._recorder_api.create_recorder(create_recorder)
        
        return response
    
    def update_recorder(self, recorder_id, status):
        update_recorder_payload = {
            "status": status,
        }
        params  = {"id": recorder_id} 
        response = self._recorder_api.update_recorder(params, update_recorder_payload)
        
        return response
    
    def list_recorders(self, page_number, page_size, filters = {} ):
        payload  = {
            "pager": {
                "page_number": page_number,
                "page_size":page_size
            },
            "filters": filters
        }
        response = self._recorder_api.list_recorders(payload)
        return response
    
    def get_recorder(self, recorder_id):
        response = self._recorder_api.get_recorder(recorder_id)
        
        return response
        
    def delete_recorder(self, recorder_id):
        params  = {
            "recorder_id": recorder_id
        }
        response = self._recorder_api.delete_recorder(params)
        return response
        
        
    class Recorder_HttpApi:
        
        def __init__(self, auth_handler, api_config = PROD_API_CONFIG):
            self._recorder_endpoint = '/gateway/recorder'
            self._http_service = HttpService(api_config=api_config)
            
            self._headers =  {
                "Authorization": "Bearer " + auth_handler.get_access_token()
            }
       
        def get_headers(self):
            return copy.deepcopy(self._headers)
            
        def create_recorder(self, payload):
            endpoint = self._recorder_endpoint
            response = self._http_service.POST_request(endpoint, payload, self.get_headers())
            
            return response
        
        def update_recorder(self, params, payload):
            endpoint = self._recorder_endpoint
            response = self._http_service.PUT_request(endpoint, payload, params, self.get_headers())
            
            return response
        
        def list_recorders(self, payload ):
            endpoint = self._recorder_endpoint + "/list"
            request_headers = self.get_headers()
            request_headers["content-type"] = "application/json"
            response = self._http_service.POST_request(endpoint, payload, request_headers)
            
            return response
        
        def get_recorder(self, recorder_id):

            endpoint = self._recorder_endpoint + "/" + str(recorder_id)
            response = self._http_service.GET_request(endpoint, {}, self.get_headers())
            
            return response
            
        def delete_recorder(self, params):
            endpoint = self._recorder_endpoint
            response = self._http_service.DELETE_request(endpoint, params, self.get_headers())
            
            return response


import copy
from datetime import datetime
from neurospeed.utils.http_service import HttpService
import json
from constants import PROD_API_CONFIG



class Insight_User_Handler:
    """
    Handles user insights by interacting with the API via HTTP requests.
    Provides methods to retrieve various types of insights for a given user.
    """
    def __init__(self, customer_auth_handler, api_config = PROD_API_CONFIG) -> None:
        """
        Initializes the Insight_User_Handler.
        """
        self._insight_api = self.Insight_HttpApi(customer_auth_handler, api_config)

    def get_user_insights(self, user_id):
        """
        Retrieves basic user insights for a specified user.

        Args:
            user_id (int): The ID of the user whose insights are to be fetched.

        Returns:
            response: The response from the API containing user insights data.
        """
        params = {"user_id":user_id}
        response = self._insight_api.get_user_insights(params)

        return response
    
    def get_user_alltime_insights(self, user_id, start_date, end_date, insight_type, boundry):
        """
        Retrieves all-time insights for a user within a specific date range.

        Args:
            user_id (int): The ID of the user.
            start_date (str): The start date for the data range (format: YYYY-MM-DD).
            end_date (str): The end date for the data range (format: YYYY-MM-DD).
            insight_type (str): The type of insight data to be retrieved.
            boundry (str): The time boundary for the data (e.g., 'hour', 'day').

        Returns:
            response: The response from the API containing all-time insights.
        """
        params = {
            "user_id": user_id,
            "start_date":start_date,
            "end_date":end_date,
            "insight_path":insight_type,
            "boundry":boundry
        }
        
        return self._insight_api.get_user_alltime_insights(params)
    
    def get_user_insights_stats(self, user_id, daily_date, insight_type, boundry):
        """
        Retrieves statistical insights for a specific day for a user.

        Args:
            user_id (int): The ID of the user.
            daily_date (str): The date for which insights are required (format: YYYY-MM-DD).
            insight_type (str): The type of insight data to be retrieved.
            boundry (str): The time boundary for the data (e.g., 'hour', 'day').

        Returns:
            response: The response from the API containing daily statistical insights.
        """

        params = {
            "user_id": user_id,
            "daily_date":daily_date,
            "insight_id":insight_type,
            "time_type":boundry
        }
        
        return self._insight_api.get_user_insights_stats(params)

    def get_user_stress_baseline(self, insight_type,user_id = None):
        params = {
            "user_id": user_id,
            "insight_id": insight_type
        }
        
        return self._insight_api.get_user_stress_baseline(params)
    
    def get_user_daily_stress(self, daily_date, timezone = None, user_id = None):
        params = {
            "user_id": user_id,
            "daily_date":daily_date,
            "local_timezone":timezone
        }
        
        return self._insight_api.get_user_daily_stress(params)
    
    def get_user_battery(self, daily_date, timezone = None, user_id = None):

        params = {
            "user_id": user_id,
            "daily_date":daily_date,
            "local_timezone":timezone
        }
        
        return self._insight_api.get_user_battery(params)
    
    def get_user_hourly_insights(self, daily_date, timezone = None, user_id = None):
        
        params = {
            "user_id": user_id,
            "daily_date":daily_date,
            "local_timezone":timezone
        }
        
        return self._insight_api.get_user_hourly_insights(params)
    
    def get_user_history_hourly_insights(self, timezone = None, user_id = None):
        params = {
            "user_id": user_id,
            "local_timezone":timezone
        }
        
        return self._insight_api.get_user_history_hourly_insights(params)
    
    def get_weekly_stress_average(self, daily_date, timezone = None, user_id = None):
        params = {
            "user_id": user_id,
            "daily_date":daily_date,
            "local_timezone":timezone
        }
        
        return self._insight_api.get_weekly_stress_average(params)
    
    def get_weekly_stress_change(self, daily_date, timezone = None, user_id = None):
        params = {
            "user_id": user_id,
            "daily_date":daily_date,
            "local_timezone":timezone
        }
        
        return self._insight_api.get_weekly_stress_change(params)
    
    def get_weekly_stress_index(self, daily_date, timezone = None, user_id = None):
        params = {
            "user_id": user_id,
            "daily_date":daily_date,
            "local_timezone":timezone
        }
        
        return self._insight_api.get_weekly_stress_index(params)
    
    def get_weekly_best_sleep(self, daily_date, user_id = None):
        params = {
            "user_id": user_id,
            "daily_date":daily_date
        }
        
        return self._insight_api.get_weekly_best_sleep(params)
    
    def get_weekly_worst_sleep(self, daily_date, user_id = None):
        params = {
            "user_id": user_id,
            "daily_date":daily_date
        }
        
        return self._insight_api.get_weekly_worst_sleep(params)
    
    def get_weekly_stressful_time(self, daily_date, timezone = None, user_id = None):
        params = {
            "user_id": user_id,
            "daily_date":daily_date,
            "local_timezone":timezone
        }
        
        return self._insight_api.get_weekly_stressful_time(params)
    
    def get_weekly_quiet_time(self, daily_date, timezone = None, user_id = None):
        params = {
            "user_id": user_id,
            "daily_date":daily_date,
            "local_timezone":timezone
        }
        
        return self._insight_api.get_weekly_quiet_time(params)
    
    
    def get_user_all_insights_stats(self, user_id, daily_date, insight_type, boundry):
        """
        Retrieves all available statistical insights for a specific day for a user.

        Args:
            user_id (int): The ID of the user.
            daily_date (str): The date for which insights are required (format: YYYY-MM-DD).
            insight_type (str): The type of insight data to be retrieved.
            boundry (str): The time boundary for the data (e.g., 'hour', 'day').

        Returns:
            response: The response from the API containing all daily statistical insights.
        """
        params = {
            "user_id": user_id,
            "daily_date":daily_date,
            "insight_id":insight_type,
            "time_type":boundry
        }
        
        return self._insight_api.get_user_all_insights_stats(params)

    class Insight_HttpApi:
        """
        A helper class responsible for making HTTP requests to the API endpoints
        related to user insights.
        """
        def __init__(self,auth_handler, api_config = PROD_API_CONFIG) -> None:
            """
            Initializes the Insight_HttpApi.
            """
            self._insight_endpoint = '/users/settings'
            self._http_service = HttpService(api_config=api_config)
            self._headers =  {
                "Authorization": "Bearer " + auth_handler.get_access_token(),             
            }
        
        def get_user_insights(self,params):
            """
            Sends a GET request to retrieve user insights.

            Args:
                params (dict): Parameters for the API request.

            Returns:
                response: The response from the API.
            """
            endpoint = self._insight_endpoint + "/insights"
            response = self._http_service.GET_request(endpoint,params,self.get_headers())
            return response
        
        def get_user_alltime_insights(self,params):
            """
            Sends a POST request to retrieve all-time user insights.

            Args:
                params (dict): Parameters for the API request.

            Returns:
                response: The response from the API.
            """
            endpoint = self._insight_endpoint+"/insights/alltime"
            response = self._http_service.POST_request(endpoint,params,self.get_headers())
            return response

        def get_user_insights_stats(self,params):
            """
            Sends a POST request to retrieve user insights statistics for a specific day.

            Args:
                params (dict): Parameters for the API request.

            Returns:
                response: The response from the API.
            """
            endpoint = self._insight_endpoint+"/insights/daily_stats"
            response = self._http_service.POST_request(endpoint,payload=params,headers=self.get_headers())
            return response
        
        def get_user_stress_baseline(self,params):
            endpoint = self._insight_endpoint+"/insights/stress_baseline"
            response = self._http_service.POST_request(endpoint,payload=params,headers=self.get_headers())
            return response
        
        def get_user_daily_stress(self,params):
            endpoint = self._insight_endpoint+"/insights/daily_stress"
            response = self._http_service.POST_request(endpoint,payload=params,headers=self.get_headers())
            return response
        
        def get_user_battery(self,params):
            endpoint = self._insight_endpoint+"/insights/battery"
            response = self._http_service.POST_request(endpoint,payload=params,headers=self.get_headers())
            return response
        
        def get_user_hourly_insights(self,params):
            endpoint = self._insight_endpoint+"/insights/hourly_insights"
            response = self._http_service.POST_request(endpoint,payload=params,headers=self.get_headers())
            return response
        
        def get_user_history_hourly_insights(self,params):
            endpoint = self._insight_endpoint+"/insights/history_hourly_insights"
            response = self._http_service.POST_request(endpoint,payload=params,headers=self.get_headers())
            return response
        
        def get_weekly_stress_average(self,params):
            endpoint = self._insight_endpoint+"/insights/weekly_stress_average"
            response = self._http_service.POST_request(endpoint,payload=params,headers=self.get_headers())
            return response
        
        def get_weekly_stress_change(self,params):
            endpoint = self._insight_endpoint+"/insights/weekly_stress_change"
            response = self._http_service.POST_request(endpoint,payload=params,headers=self.get_headers())
            return response
        
        def get_weekly_stress_index(self,params):
            endpoint = self._insight_endpoint+"/insights/weekly_stress_index"
            response = self._http_service.POST_request(endpoint,payload=params,headers=self.get_headers())
            return response
        
        def get_weekly_best_sleep(self,params):
            endpoint = self._insight_endpoint+"/insights/weekly_best_sleep"
            response = self._http_service.POST_request(endpoint,payload=params,headers=self.get_headers())
            return response
        
        def get_weekly_worst_sleep(self,params):
            endpoint = self._insight_endpoint+"/insights/weekly_worst_sleep"
            response = self._http_service.POST_request(endpoint,payload=params,headers=self.get_headers())
            return response
        
        def get_weekly_stressful_time(self,params):
            endpoint = self._insight_endpoint+"/insights/weekly_stressful_time"
            response = self._http_service.POST_request(endpoint,payload=params,headers=self.get_headers())
            return response
        
        def get_weekly_quiet_time(self,params):
            endpoint = self._insight_endpoint+"/insights/weekly_quiet_time"
            response = self._http_service.POST_request(endpoint,payload=params,headers=self.get_headers())
            return response
        
        def get_user_all_insights_stats(self,params):
            """
            Sends a POST request to retrieve all user insights statistics.

            Args:
                params (dict): Parameters for the API request.

            Returns:
                response: The response from the API.
            """
            endpoint = self._insight_endpoint+"/insights/all_daily_stats"
            response = self._http_service.POST_request(endpoint,params,self.get_headers())
            return response
        
        def get_headers(self):
            """
            Retrieves a copy of the authorization headers.

            Returns:
                dict: A deep copy of the authorization headers.
            """
            return copy.deepcopy(self._headers)


#from neurospeed.auth.auth_as_user_handler import Auth_AS_User_Handler
#from neurospeed.utils.helper_service import UtilService

from neurospeed.hia_user_data.neurobrave_sensor_interface import HIA_Client

from neurospeed.auth.auth_as_user_handler import Auth_AS_User_Handler
from neurospeed.auth.auth_as_customer_handler import Auth_AS_Customer_handler
from neurospeed.api_http_handlers.recorder_http_handler import UserRoom_Recorder_Handler
from neurospeed.api_http_handlers.exporter_http_handler import UserRoom_Recorder_Exporter_Handler
from neurospeed.api_http_handlers.insight_http_handler import Insight_User_Handler
from neurospeed.api_http_handlers.customer_account_manager import Customer_Account_Manager
from neurospeed.utils.helper_service import UtilService
from neurospeed.api_http_handlers.customer_account_manager import Customer_Account_Manager

import json
import numpy as np
import time 
import logging
import requests
from datetime import datetime
import os
from dateutil import tz
import pytz
from constants import PROD_API_CONFIG

global recorder_handler
global exporter_handler


def find_running_recorder(customer_auth, username, api_config = PROD_API_CONFIG):
    global recorder_handler
    recorder_handler = UserRoom_Recorder_Handler(customer_auth, api_config=api_config)
    list_recorders = recorder_handler.list_recorders(1, 50, {"username": username})
        
    if list_recorders is not None:
        for index, item in enumerate(list_recorders["pager"]["items"]):
            if not list_recorders["pager"]["items"][index]["status"] == "stopped":
                logging.debug("discovered Recorder with status not stopped, stopping recorder")                
                rec_id = list_recorders["pager"]["items"][index]["id"]                
                return rec_id
    return None

def run_recorder_flow(customer_auth, hia_config, recorder_name = "default"):
    '''
    create data recorder instance on cloud.
    recorder created with status "resource_pending".
    once available recording resources found and assgined
    recorder status will change to "pending",
    then once recorder picked by assigned recorder resources and start to record status will change to "recording"
    '''
    global recorder_handler
     
    username = hia_config["username"]
    # set up data recorder instance:
    # list_recorders = recorder_handler.list_recorders(0,50)
    
    list_recorders = recorder_handler.list_recorders(1, 50, {"username": username})
    logging.debug("list of existing recorders:")
    if type(list_recorders) == dict:
        logging.debug(list_recorders.keys())
    if list_recorders is not None:
        for index, item in enumerate(list_recorders["pager"]["items"]):
            if not list_recorders["pager"]["items"][index]["status"] == "stopped":
                logging.debug("discovered Recorder with status not stopped, stopping recorder")                
                rec_id = list_recorders["pager"]["items"][index]["id"]
                recorder_handler.update_recorder(rec_id, "stopped")                 
                while True:                    
                    if recorder_handler.get_recorder(rec_id)["status"] == "stopped":
                        logging.debug("stopped recorder successfully")
                        time.sleep(5)
                        break


    recorder = recorder_handler.create_recorder(username = username, recorder_name = recorder_name)
    if recorder == None:
        logging.debug("failed to create recorder")
        return None
        
    recorder_id = str(recorder["id"])
    logging.debug('created recorder with id' + recorder_id)
    # get recorder ID for further administration:
    recorder = recorder_handler.get_recorder(recorder_id)     
    logging.debug("recorder status: " + recorder["status"])
    
    # wait until cloud resources allocated to recorder:
    recorder_recording = False
    max_query_attemps = 5
    query_attemps = 1
    while (recorder_recording != True & (query_attemps < max_query_attemps) ):
        recorder = recorder_handler.get_recorder(recorder_id)   
        logging.debug("recorder status: " + recorder["status"])
        if (recorder["status"] == "recording"):
            logging.debug('recorder started recording')
            recorder_recording = True
        else:
            logging.debug('recorder not recording yet')
            query_attemps = query_attemps + 1
            time.sleep(10)
             
    if (recorder_recording != True):
        logging.debug('recorder not yet recording after ' + str(query_attemps)  + ' query attempts')
        recorder_handler.delete_recorder(recorder_id)
        return None
     

    # #once recorder starts recording, you can change it's status to paused or stopped. you can resume paused recorder but not stopped recorder
    # # pause recorder:
    # res = recorder_handler.update_recorder(recorder_id, "paused")
    # recorder = recorder_handler.get_recorder(recorder_id)     
    # print("recorder status: " + recorder["status"])
    # time.sleep(15)
    
    # # reactivate paused recorder
    # recorder_handler.update_recorder(recorder_id, "pending") 
    # recorder = recorder_handler.get_recorder(recorder_id)     
    # print("recorder status: " + recorder["status"])
 
    #recorder_handler.delete_recorder(recorder_id) # uncomment this to delete recorder
    # recorders = recorder_handler.list_recorders(username)
    # print(recorders)
    
     
    
    
    return recorder_id


def start_recording(customer_auth, hia_config, recorder_name, api_config = PROD_API_CONFIG):
    global recorder_handler
    recorder_handler = UserRoom_Recorder_Handler(customer_auth, api_config=api_config)   
    recorder_id = run_recorder_flow(customer_auth, hia_config, recorder_name)
    
    while recorder_id is None:
        print("FAILED starting the cloud recording serivce! Retrying.\n")
        recorder_id = run_recorder_flow(customer_auth, hia_config, recorder_name)
    return recorder_id


def get_user_insights(customer_auth,username, api_config = PROD_API_CONFIG):
    insight_handler = Insight_User_Handler(customer_auth, api_config)
    cam = Customer_Account_Manager(customer_auth, api_config)
    user_id = cam.get_user_id(username)
    insights = insight_handler.get_user_insights(user_id)
    return insights


def get_user_alltime_insights(customer_auth,username,start_date,end_date,insight_type="output$neurobrave_stress2",boundry="hour", api_config = PROD_API_CONFIG):
    insight_handler = Insight_User_Handler(customer_auth, api_config)
    cam = Customer_Account_Manager(customer_auth, api_config)
    user_id = cam.get_user_id(username)
    
    insights = insight_handler.get_user_alltime_insights(user_id,start_date,end_date,insight_type,boundry)
    return insights


def get_user_insights_stats(customer_auth,username,daily_date,insight_type="output$neurobrave_stress2",boundry="day", api_config = PROD_API_CONFIG):
    """
    The function will return the statistics for date_day_day only.
    boundry can be day/week/month.
    """
    insight_handler = Insight_User_Handler(customer_auth, api_config)
    cam = Customer_Account_Manager(customer_auth, api_config)
    user_id = cam.get_user_id(username)
    insights = insight_handler.get_user_insights_stats(user_id,daily_date,insight_type,boundry)
    
    return insights

def get_user_all_insights_stats(customer_auth,username,daily_date,insight_type="output$neurobrave_stress2",boundry="day", api_config = PROD_API_CONFIG):
    """
    The function will return the statistics of all dates up to and including daily_date
    boundry can be hour/day/week/month.
    """
    insight_handler = Insight_User_Handler(customer_auth, api_config)
    cam = Customer_Account_Manager(customer_auth, api_config)
    user_id = cam.get_user_id(username)
    
    insights = insight_handler.get_user_all_insights_stats(user_id,daily_date,insight_type, boundry)
    return insights

def get_user_stress_baseline(auth, username = None, api_config = PROD_API_CONFIG, insight_type="output$neurobrave_stress2"):
    """
    The function will return the daily stress
    """
    insight_handler = Insight_User_Handler(auth, api_config)
    if type(auth) == Auth_AS_Customer_handler:
        cam = Customer_Account_Manager(auth, api_config)
        user_id = cam.get_user_id(username)
        insights = insight_handler.get_user_stress_baseline(insight_type, user_id)
    else:
        insights = insight_handler.get_user_stress_baseline(insight_type)
    
    return insights

def get_user_daily_stress(auth, daily_date, timezone = None, username = None, api_config = PROD_API_CONFIG):
    """
    The function will return the daily stress
    """
    insight_handler = Insight_User_Handler(auth, api_config)
    if type(auth) == Auth_AS_Customer_handler:
        cam = Customer_Account_Manager(auth, api_config)
        user_id = cam.get_user_id(username)
        insights = insight_handler.get_user_daily_stress(daily_date, timezone, user_id)
    else:
        insights = insight_handler.get_user_daily_stress(daily_date, timezone)
    
    return insights

def get_user_battery(auth, daily_date, timezone = None, username = None, api_config = PROD_API_CONFIG):
    """
    The function will return the daily stress
    """
    insight_handler = Insight_User_Handler(auth, api_config)
    if type(auth) == Auth_AS_Customer_handler:
        cam = Customer_Account_Manager(auth, api_config)
        user_id = cam.get_user_id(username)
        insights = insight_handler.get_user_battery(daily_date, timezone, user_id)
    else:
        insights = insight_handler.get_user_battery(daily_date, timezone)
    
    return insights

def get_user_hourly_insights(auth, daily_date,timezone = None, username = None, api_config = PROD_API_CONFIG):
    insight_handler = Insight_User_Handler(auth, api_config)
    
    if type(auth) == Auth_AS_Customer_handler:
        cam = Customer_Account_Manager(auth, api_config)
        user_id = cam.get_user_id(username)
        insights = insight_handler.get_user_hourly_insights(daily_date, timezone, user_id)
    else:
        insights = insight_handler.get_user_hourly_insights(daily_date, timezone)
    
    return insights

def get_user_history_hourly_insights(auth, timezone = None, username = None, api_config = PROD_API_CONFIG):
    insight_handler = Insight_User_Handler(auth, api_config)
    
    if type(auth) == Auth_AS_Customer_handler:
        cam = Customer_Account_Manager(auth, api_config)
        user_id = cam.get_user_id(username)
        insights = insight_handler.get_user_history_hourly_insights(timezone, user_id)
    else:
        insights = insight_handler.get_user_history_hourly_insights(timezone)
        
    return insights

def get_weekly_stress_average(auth, daily_date, timezone = None, username = None, api_config = PROD_API_CONFIG):
    """
    The function will return the weekly stress average
    """
    insight_handler = Insight_User_Handler(auth, api_config)
    if type(auth) == Auth_AS_Customer_handler:
        cam = Customer_Account_Manager(auth, api_config)
        user_id = cam.get_user_id(username)
        insights = insight_handler.get_weekly_stress_average(daily_date, timezone, user_id)
    else:
        insights = insight_handler.get_weekly_stress_average(daily_date, timezone)
    
    return insights

def get_weekly_stress_change(auth, daily_date, timezone = None, username = None, api_config = PROD_API_CONFIG):
    """
    The function will return the weekly stress change
    """
    insight_handler = Insight_User_Handler(auth, api_config)
    if type(auth) == Auth_AS_Customer_handler:
        cam = Customer_Account_Manager(auth, api_config)
        user_id = cam.get_user_id(username)
        insights = insight_handler.get_weekly_stress_change(daily_date, timezone, user_id)
    else:
        insights = insight_handler.get_weekly_stress_change(daily_date, timezone)
    
    return insights

def get_weekly_stress_index(auth, daily_date, timezone = None, username = None, api_config = PROD_API_CONFIG):
    """
    The function will return the weekly stress index
    """
    insight_handler = Insight_User_Handler(auth, api_config)
    if type(auth) == Auth_AS_Customer_handler:
        cam = Customer_Account_Manager(auth, api_config)
        user_id = cam.get_user_id(username)
        insights = insight_handler.get_weekly_stress_index(daily_date, timezone, user_id)
    else:
        insights = insight_handler.get_weekly_stress_index(daily_date, timezone)
    
    return insights
    
def get_weekly_best_sleep(auth, daily_date, username = None, api_config = PROD_API_CONFIG):
    """
    The function will return the night and the value of the best sleep
    """
    insight_handler = Insight_User_Handler(auth, api_config)
    if type(auth) == Auth_AS_Customer_handler:
        cam = Customer_Account_Manager(auth, api_config)
        user_id = cam.get_user_id(username)
        insights = insight_handler.get_weekly_best_sleep(daily_date, user_id)
    else:
        insights = insight_handler.get_weekly_best_sleep(daily_date)
    
    return insights

def get_weekly_worst_sleep(auth, daily_date, username = None, api_config = PROD_API_CONFIG):
    """
    The function will return the night and the value of the best sleep
    """
    insight_handler = Insight_User_Handler(auth, api_config)
    if type(auth) == Auth_AS_Customer_handler:
        cam = Customer_Account_Manager(auth, api_config)
        user_id = cam.get_user_id(username)
        insights = insight_handler.get_weekly_worst_sleep(daily_date, user_id)
    else:
        insights = insight_handler.get_weekly_worst_sleep(daily_date)
    
    return insights
       
def get_weekly_stressful_time(auth, daily_date, timezone = None, username = None, api_config = PROD_API_CONFIG):
    """
    The function will return the most stressful time in the week
    """
    insight_handler = Insight_User_Handler(auth, api_config)
    if type(auth) == Auth_AS_Customer_handler:
        cam = Customer_Account_Manager(auth, api_config)
        user_id = cam.get_user_id(username)
        insights = insight_handler.get_weekly_stressful_time(daily_date, timezone, user_id)
    else:
        insights = insight_handler.get_weekly_stressful_time(daily_date, timezone)
    
    return insights    

def get_weekly_quiet_time(auth, daily_date, timezone = None, username = None, api_config = PROD_API_CONFIG):
    """
    The function will return the most stressful time in the week
    """
    insight_handler = Insight_User_Handler(auth, api_config)
    if type(auth) == Auth_AS_Customer_handler:
        cam = Customer_Account_Manager(auth, api_config)
        user_id = cam.get_user_id(username)
        insights = insight_handler.get_weekly_quiet_time(daily_date, timezone, user_id)
    else:
        insights = insight_handler.get_weekly_quiet_time(daily_date, timezone)
    
    return insights    
    
    
    
def delete_recording(recorder_id):
    recorder_handler.delete_recorder(recorder_id) 
    print("recorder status: " + recorder_handler.get_recorder(recorder_id)["status"])


def stop_recording(recorder_id):
    recorder_handler.update_recorder(recorder_id, "stopped") 
    print("recorder status: " + recorder_handler.get_recorder(recorder_id)["status"])


def run_exporter_flow(recorder_id, exporter_config, hia_config, save_folder, max_query_attemps = 600):
    '''
    inputs:
        recorder id - string with recorder id in neurospeed that's used in this section
        exporter_config - dictionry with exported configuration. exporter name, time limits, fields to export.
        hia_config - dictionary with hia_config. used for the credentials.
        save_folder - absolute folder path to save the .csv file
    
    outputs:
        full absolute path to downloaded .csv file with data
        
        
    this function operates the exported on neurospeed via the neurospeed API.
    it runs exporter on cloud, waits for it to complete, and downloads the file using received temporary URL. 
    the URL deprecated quickly so don't expect it to work after even few minutes of completing the export. 
    
    the downloaded file is .csv, same format like downlaoding from exporter via neurospeed dashboard
    
    '''
    
    global exporter_handler
    
    if not os.path.isdir(save_folder):
        os.mkdir(save_folder)
        
    # create exporter, if recorder has no data, request will fail.
    exporter = exporter_handler.create_exporter(int(recorder_id), exporter_config)
    exporter_id = str(exporter["id"])

    exporter = exporter_handler.get_exporter(exporter_id)  
    logging.debug("EXPORTER: ")
    logging.debug(exporter)

    # once exporter created, query it's status until it "exported".
    for query_attempt in range(max_query_attemps):
        exporter = exporter_handler.get_exporter(exporter_id)  
        logging.debug("exporter status: " + exporter["status"])
        if (exporter["status"] == "exported"):
            logging.debug('exporter finished exporting')
            break
        time.sleep(2)
             
        
    if exporter["status"] != "exported":
        logging.debug('exporter not yet exported the file after  ' + str(max_query_attemps)  + ' query attempts')
        exporter_handler.delete_exporter(exporter_id)
        return None
   
    # once exporter exported the file, you can ask for an URL to download the exported file. 
    #the url available for only short time before it deprecates
    #get_exported_url(exporter_id) returns dictionary of structure {"url": "https://neurospeed...... ...."}
    url_response = exporter_handler.get_exported_url(exporter_id)["url"]  
    #delete_exporter(exporter_id) # uncomment this to delete exporter
    # list all exporters under username
    # exporters = exporter_handler.list_exporters(hia_config["username"])
    # print(exporters)    
    logging.debug("data download link: " + url_response)
    
    exported_filename = None
    try:
        print("downloading data from server, ignoring TLS certificate")
        exported_data = requests.get(url_response, verify = False)
        exported_filename = save_folder + "/_" + exporter_config["custom_name"] + datetime.now().strftime("%d-%m-%Y %H-%M-%S") + '.csv'
        logging.debug("saving data to local file at " + exported_filename)
        with open(exported_filename, "wb")as f:
            f.write(exported_data.content)
    except Exception as e:
        logging.debug("failed to download the datafile, please attempt manual downlaod from the web dashboard")
        logging.debug(e)
    return exported_filename
    
    
def download_data(customer_auth, recorder_id, exporter_config, hia_config, save_folder, max_query_attemps = 600, api_config = PROD_API_CONFIG):
    
    '''
    this function tries to download the CSV datafile from neurospeed cloud,
    the download will fail in case of non-exsiting recording or in case that donwloads from AWS cloud are blocked by the network
    
    returns full absolute path to the downloaded data file on local computer
    
    '''
    global exporter_handler
    exporter_handler = UserRoom_Recorder_Exporter_Handler(customer_auth, api_config=api_config)
    exported_filename = run_exporter_flow(recorder_id, exporter_config, hia_config, save_folder, max_query_attemps)
    return exported_filename


def get_all_existing_users(customer_auth, page_number = 1, page_size = 50, api_config = PROD_API_CONFIG):
    '''
    This function returns a list of all existing users associated with a customer account.
    
    Parameters:
    customer_auth (object): An authentication object that provides access to the customer account.
    
    Returns:
    list: A list of all existing users associated with the customer account. Each user in the list contains
          relevant user information, as provided by the `get_all_existing_users` method of `Customer_Account_Manager`.
    
    '''
    # Get the list of all existing users
    customer_account = Customer_Account_Manager(customer_auth, api_config=api_config)
    existing_users = customer_account.get_all_existing_users(page_number, page_size)
    return existing_users["pager"]["items"]


def generate_sensor_info(config, hia_config, use_local_timestamp = False):
    '''
    this functiuon generates sensor_info dictionary that is nesessary ot initialize the HIA client.
    the sensor info is tailored for single EEG according to config
    only LSL interface supported currently

    input:
        hia_config dictionary
    '''
    # generate SOME sensors stream ids
    sensor_info = dict()

    if "EEG" in config["DATA_SOURCE"]:
        user_data_stream_id = "EEG" + '_' + UtilService.generateId(6)
        sensor_info[user_data_stream_id] = {
            "device_type": "EEG",
            "channel_count": len(hia_config["EEG_channel_enable"]),
            "sampling_frequency": hia_config["EEG_sample_rate"],
            "buffer_length": hia_config["EEG_buffer_length_seconds"],
            "manufacturer_id": "NeuroBrave",
            "sensor_id": user_data_stream_id,
            "stream_id": user_data_stream_id,
            "stream_state": "enabled",
            "channel_map": hia_config["channel_map"]
        }
        
        if use_local_timestamp: 
            sensor_info[user_data_stream_id]["channel_map"] += ["timestamp"]
            

    if "PPG" in config["DATA_SOURCE"]:
        user_data_stream_id = "PPG" + '_' + UtilService.generateId(6)
        sensor_info[user_data_stream_id] = {
            "device_type": "PPG",
            "channel_count": len(hia_config["PPG_channel_map"]),
            "sampling_frequency": hia_config["PPG_sample_rate"],
            "buffer_length": hia_config["PPG_buffer_length_seconds"],
            "manufacturer_id": "NeuroBrave",
            "sensor_id": user_data_stream_id,
            "stream_id": user_data_stream_id,
            "stream_state": "enabled",
            "channel_map": hia_config["PPG_channel_map"]
        }
        if use_local_timestamp: 
            sensor_info[user_data_stream_id]["channel_map"] += ["timestamp"]

    if "SMARTWATCH" in config["DATA_SOURCE"]:
        user_data_stream_id = "SMARTWATCH" + '_' + UtilService.generateId(6)
        sensor_info[user_data_stream_id] = {
            "device_type": "SMARTWATCH",
            "channel_count": len(hia_config["PPG_channel_map"]),
            "sampling_frequency": hia_config["PPG_sample_rate"],
            "buffer_length": hia_config["PPG_buffer_length_seconds"],
            "manufacturer_id": "NeuroBrave",
            "sensor_id": user_data_stream_id,
            "stream_id": user_data_stream_id,
            "stream_state": "enabled",
            "channel_map": hia_config["PPG_channel_map"]
        }
        if use_local_timestamp: 
            sensor_info[user_data_stream_id]["channel_map"] += ["timestamp"]


    if "GSR" in config["DATA_SOURCE"]:
        for idx, port in enumerate(config["OPENBCI_COM_PORT"]):
            user_data_stream_id = "GSR" + '_' + UtilService.generateId(6)
            sensor_info[user_data_stream_id] = {"device_type": "GSR",
                                                "sensor_id": user_data_stream_id,
                                                "stream_id": user_data_stream_id,
                                                "sampling_frequency": 250,
                                                "buffer_length": 1.0,
                                                "manufacturer_id": "openBCI",
                                                "channel_count": len(config["GSR_channel_enable"][idx]),
                                                # "Downstream ID":downstream_id,
                                                "channel_map": config["GSR_channel_map"][idx],
                                                "stream_state": "enabled"}
            
            if use_local_timestamp: 
                sensor_info[user_data_stream_id]["channel_map"] += ["timestamp"]

    if "EDA" in config["DATA_SOURCE"]:
        user_data_stream_id = "GSR_ADIEDA"

        sensor_info[user_data_stream_id] = {"device_type": "GSR",
                                            "sensor_id": user_data_stream_id,
                                            "stream_id": user_data_stream_id,
                                            "sampling_frequency": config["EDA_sampling_frequency"],
                                            "buffer_length": 1.0,
                                            "manufacturer_id": "ADI",
                                            "channel_count": 6,
                                            # "Downstream ID":downstream_id,
                                            "channel_map": ['real', 'img', 'adm_real', 'adm_img', 'adm_mag',
                                                            'adm_phase', "timestamp"],
                                            "stream_state": "enabled"}
        if use_local_timestamp: 
            sensor_info[user_data_stream_id]["channel_map"] += ["timestamp"]
         
            
    logging.debug(f"generated sensor info {sensor_info}")
    return sensor_info


def init_HIA_client(config, hia_config, use_local_timestamp = False, api_config=PROD_API_CONFIG):
    '''
    this functiuon  initialize the HIA client and connect the cloud streaming (upload raw data to neurospeed)
    the sensor info is tailored for single EEG according to config
    only LSL interface supported currently

    input:
        config - dictionary with config for this epxeriment program
        hia_config dictionary

    output
        hia_user client object instance
        sensor info dictionary

    '''
    retries=3
    for _ in range(retries):
        try:
            user1_auth = Auth_AS_User_Handler(hia_config)
            user1_auth.login()

            hia_sensor_info_user = generate_sensor_info(config, hia_config, use_local_timestamp)

            logging.debug('Generated sensor info: {}'.format(hia_sensor_info_user))
            hia_instance = HIA_Client(user1_auth, hia_sensor_info_user, api_config=api_config)
            
            #hia_user1.set_socket_connection_external_handler(connection_external_handler)
            # hia_user1.set_socket_disconnect_external_handler(disconnect_external_handler)  
            logging.debug("HIA_ID: " + str(user1_auth.get_hia_id()))
            hia_instance._device_type = "EEG"
            hia_instance.connect()
            break       
        except Exception as e:
            logging.debug(f"error while conneting HIA, retrying...: {e}")
            time.sleep(5)
    
    
    return hia_instance, hia_sensor_info_user


def get_stream_id_from_sensor_info(sensor_info, device_type = "EEG"):
    
    supported_devices = "EEG", "PPG", "SMARTWATCH", "GSR", "EDA", "USER_DATA"
    if device_type not in supported_devices:
        raise ValueError("device not supported")
    logging.debug(f"searching for {device_type} in local sensor_info..")
    stream_id = list(filter(lambda x: x.startswith(device_type), list(sensor_info.keys())))[0]
    logging.debug(f"found stream id {stream_id}")
    
    return stream_id


def send_datafile_to_neurospeed(data_array,timestamp_array = None, labels_array=None, Fs=256, channel_map = None, hia_config = None, window = None, device_type = "EEG",sleep_time=0.1):

    
    '''
    
    this function transmits data structure to neurospeed using HIA streaming, in realtime speed.
    inputs:
        data arry - a NUMPY data array of shape SAMPLES x CHANNELS
        timestamp  - NUMPY array of shape SAMPLEX X . 
        labels array - optional, numpy array of SAMPLES x 1 of strings that contain data lables
        Fs - data sampling frequency, 
        channel map - list of strings that specify channel names, must be same size as CHANNELS dimension of the data array
        hia_config- dictionary with neurospeed cloud user-level credentials - username, customer profile string, and the password
        window - optional, number of samples to send in each transmission packet
        
    '''
    
    
    
    if timestamp_array is None: 
        send_without_timestamp = True
        use_local_timestamp = False
    else: 
        send_without_timestamp = False
        use_local_timestamp = True
    
    
    hia_client = None
    
    if hia_config is None: 
        try:
            with open('config/hia_config.json') as f:
                hia_config = json.load(f)
        except:
            raise ValueError("unable to load HIA configuration file")
            
            
    device_type = device_type.upper()
    
    config = {}
    config["DATA_SOURCE"] = [device_type]  
    if window is None:
        EEG_buffer_length_seconds =  0.25
    else:
        EEG_buffer_length_seconds = window/Fs
    channel_count = np.min(data_array.shape)  
    if channel_map is None: 
        channel_map = [f"ch_{i}" for i in range(channel_count)]    
    hia_config["EEG_channel_enable"] = [i for i in range(channel_count)]
    hia_config["EEG_sample_rate"] = Fs
    hia_config["EEG_buffer_length_seconds"] = EEG_buffer_length_seconds 
    hia_config["channel_map"] = channel_map
   
    
    
    
    max_hia_reconnect_retries = 3
    for reconnect_idx in range(max_hia_reconnect_retries):
        try:
            hia_client, sensor_info = init_HIA_client(config, hia_config, use_local_timestamp)
        except Exception as e:
            logging.debug(f"exception while initializing HIA instance: {e}")
            time.sleep(3)
            continue
        if hia_client.is_connected():
            break
        else:
            logging.debug("HIA connection retrying...")
            time.sleep(3)

    if not hia_client.is_connected():
        raise ValueError("FAILED connecting to cloud streaming!\n")
        

    print("hia connection status: " + str(hia_client.is_connected()))
    
    
    
    sleep_time =sleep_time  #sleep time between packets, [seconds]
    
    
    stream_id = get_stream_id_from_sensor_info(sensor_info, device_type = device_type)
    if window is None:
        window = int(EEG_buffer_length_seconds * Fs) 
            
    if labels_array is not None:
        if not np.max(labels_array.shape) == np.max(data_array.shape):
            raise ValueError("data and event array length mismatch!")
    
    read_index = 0
    current_label = None
    #print(f"length of data: {len(data_array)}")
    print(f"shape of the array is {data_array.shape}")
    print("tranmsmitting data packets...")
    while read_index + window < len(data_array):
        data = data_array[read_index:read_index + window, :].tolist()
        timestamp = timestamp_array[read_index:read_index + window].tolist()
        #print(f"data: {type(data)}, {data}")
        read_index += window
        hia_client.send_data_direct(data, stream_id = stream_id, timestamp = timestamp, send_without_timestamp=send_without_timestamp, device_type=device_type)
        print(".", end="")
        
        time.sleep(sleep_time)
        
        if labels_array is not None:
            if labels_array[read_index] != current_label:
                hia_client.set_label(str(labels_array[read_index]))
                current_label = labels_array[read_index]
                print("")
                print(f"setting label to {current_label}")


    hia_client.disconnect()
    time.sleep(1)
    if hia_client.is_connected():
        print("HIA connection status: CONNECTED")
    else:
        print("HIA connection status: DISCONNECTED")
    

def is_exporter_datafile(file):
    #this function receives ABSOLUTE PATH to data file, and checks if the file is 
    #in fact a datafile produced by NeuroSpeed cloud recorder. 
    min_size = 1E4
    if not ".csv" in file:
        return False
    if  os.path.getsize(file) < min_size: 
        return False
    #read header only: 
    try:
        header = np.loadtxt(file, delimiter=",", max_rows=1, dtype="str").tolist()
    except:
        return False
    if not "timestamp" in header:
        return False
    if not "packets_counter" in header:
        return False
    if not "stream_id" in header:
        return False
    if not "device_type" in header:
        return False

    return True  


def find_exporter_files(folder):
    # print(f"checking {folder}")
    list_of_files=[]
    for item in os.listdir(folder):   
        if is_exporter_datafile(os.path.join(folder, item)):
            list_of_files.append(os.path.join(folder, item))
            # print(f"discovered {os.path.join(workdir, item)}")
        if os.path.isdir(os.path.join(folder, item)):
            # print(f"recursing into {os.path.join(folder, item)}")
            list_of_files+=find_exporter_files(os.path.join(folder, item))
    return list_of_files




from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
   name='neurospeed',
   version='2.3.0',
   author='NeuroBrave',
   contact_email='oleg@neurobrave.com',
   long_description = long_description,
   long_description_content_type="text/markdown",
   packages=find_packages(),
   scripts=[],
   package_data={'neurospeed': ['config/*']},
   url='https://bitbucket.org/neurobrave/neurospeed_python_api',
   classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
   ],
   license='LICENSE.txt',
   description='NeuroSpeed Python API',
   install_requires=[
       "python-socketio[client]==5.3.0"
   ],
)




# -*- coding: utf-8 -*-
"""
Created on Sat Sep 11 20:12:14 2021

@author: NeuroBrave
"""

from neurospeed.utils.http_service import HttpService
import logging
import json
from constants import PROD_API_CONFIG

class Customer_Account_Manager:
    
    def __init__(self, customer_auth_handler, api_config = PROD_API_CONFIG):
        self._customer_auth  = customer_auth_handler
        self._http_service = HttpService(api_config=api_config)
        
        if customer_auth_handler.get_access_token() is None: 
            raise ValueError("no token for customer, failed to initialize gateway")
        self._headers =  {
                "Authorization": "Bearer " + customer_auth_handler.get_access_token()
                # "Content-Type": "application/json"
        }

    def check_username_for_validity(self, username):
        if type(username) is not str:
            username = str(username)
        # if not (5 <= len(username) <= 30):
        #     logging.debug("new username must be at least 8 characters")
        #     return False
        return True
    
    def get_user_id(self,username):
        if not self.check_username_for_validity(username):
            return -1
        endpoint = "/users/"+username
       
        user = self._http_service.GET_request(endpoint,{},self._headers)
        return user["id"]

    
    def check_password_for_validity(self, password):
        if type(password) is not str:
            password = str(password)
        if len(password) < 8:
            logging.debug("new password must be at least 8 characters")
            return False
        return True
          
        
    def get_all_existing_users(self, page_number = 1, page_size = 50):
        endpoint = "/users/list"
        # endpoint = "/pages/users"
        payload = {
            "pager": {
                "page_number": page_number,
                "page_size":page_size
            },
            "filters": {}
        }
        
        header = self._headers.copy()
        header["Content-Type"] =  "application/json"

        return self._http_service.POST_request(endpoint, payload, header)
  


     
    def get_customer_details(self):
        endpoint = "/customers/current"
        return  self._http_service.GET_request(endpoint, {}, self._headers)
        '''
        response should look like that: 
            
            {'id': 45,
             'email': 'lab@neurospeed.com',
             'role': 'user',
             'firstName': 'lab',
             'is_active': True,
             'lastName': 'rador',
             'account_id': 'rador_54uL9rH4XY',
             'in_active_reason': None,
             'company_name': 'rador'}
        '''

    def create_new_user(self, new_username, new_password):
        #create user:
        endpoint = "/users"
        # new_username ='api_test_user'
        if not self.check_username_for_validity(new_username):
            return new_password
        if not self.check_password_for_validity(new_username):
            return None
        params = {"username": new_username, "password": new_password}
        return self._http_service.POST_request(endpoint, params, self._headers)

        '''
        response supposed to look like that: 
            {'id': 1298,
             'username': 'api_test_user',
             'settings': {'isDspEnabled': True,
              'userId': 1298,
              'dsp_config': None,
              'ppg_config': None,
              'garmin_config': None,
              'gsr_config': None,
              'dsp_config_source': None,
              'gsr_config_source': None,
              'active_markers': None,
              'push_user_key': None,
              'push_active': True,
              'id': 1298,
              'isAttachDataOrigin': True,
              'version': 1,
              'cognitive_type': 'moderate',
              'createdAt': '2024-03-11T16:03:29.361Z',
              'updatedAt': '2024-03-11T16:03:29.361Z'}}
        
        '''
        
        
        
    def delete_user(self, id_of_user_to_delete):
        if type(id_of_user_to_delete) is not int:
            logging.debug("failed to processed user delete request: user ID must be integer value")
            return None
        endpoint = "/users"
        params = {"id": id_of_user_to_delete}
        return  self._http_service.DELETE_request(endpoint, params, self._headers)
        #delete method returns true if successfull


    
    def modify_user(self, user_id_to_modify, new_username, new_password):
        endpoint = "/users" 
        
        if not self.check_username_for_validity(new_username):
            return new_password
        if not self.check_password_for_validity(new_username):
            return None
        
        params = {"username": new_username, "password": new_password}
        
        return  self._http_service.PUT_request(endpoint=endpoint, payload=params, params={'id':user_id_to_modify}, headers = self._headers)


# -*- coding: utf-8 -*-
"""
Created on Mon Sep 13 12:42:25 2021

@author: NeuroBrave
"""

import socketio 
from neurospeed.utils.api_config_service import ApiConfig
from constants import PROD_API_CONFIG

class DownlinkRoom_AS_User_Handler:
    
    def __init__(self, user_auth_handler, api_config = PROD_API_CONFIG):
        self._contex = "DownlinkRoom_AS_User_Handler - "
        
        self._auth_handler = user_auth_handler
        self._username = self._auth_handler.get_username()
        self._connection_status = False
        self._downlink_router_external_handler = None
        
        # Create User_Downlink_Api and pass user configuration
        self._downlink_api = self.DownlinkRoom_Api(self._auth_handler, api_config=api_config)
        
    def downlinkRoom_connection_handler(self):
        print("{} User connected to user's {} DownlinkRoom".format(self._contex, self._username ))
        self._connection_status = True
    
    
    def downlinkRoom_disconnect_handler(self):
        print("{} User disconnected from user's {} DownlinkRoom".format(self._contex,self._username ))
        self._connection_status = False
    
    
    def downlinkRoom_router_handler(self, payload):
        print("{} message {} routed to user: {}".format(self._contex, payload, self._username ))

        # propogate routed message to main program
        if (self._downlink_router_external_handler != None):
            self._downlink_router_external_handler(self, payload) # send current contex along with payload (current DownlinkRoom_AS_User_Handler instance) 


    def set_downlink_router_external_handler(self, handler):
        self._downlink_router_external_handler = handler
    
    def route_message(self, payload):
        self._downlink_api.route_message(payload)
        
    def get_username(self):
        return self._username
    
    def connect(self):
        # attach relevant handlers for socket.io events
        self._downlink_api.set_connection_handler(self.downlinkRoom_connection_handler)
        self._downlink_api.set_disconnect_handler(self.downlinkRoom_disconnect_handler)
        self._downlink_api.set_downlink_handler(self.downlinkRoom_router_handler)

        # connect 
        self._downlink_api.connect()
        
    def disconnect(self):
        self._downlink_api.disconnect()
        
    def is_connected(self):
        return self._connection_status
    
    
    class DownlinkRoom_Api:
    
        def __init__(self, auth_handler, api_config = PROD_API_CONFIG):
            api_config = ApiConfig(api_config)
            
        
            self._access_token = auth_handler.get_access_token()
            self._username = auth_handler.get_username()
            self._socket_url = api_config.get_socket_url()
            
            logger_on = True
            if auth_handler.is_verbose_log() == False:
              logger_on = False
            self._sio = socketio.Client(logger=logger_on, engineio_logger=False, reconnection_delay  = 5, reconnection = True) 


            
        def set_connection_handler(self, handler):
            self._sio.on('connect',handler = handler)
            
        def set_disconnect_handler(self, handler):
            self._sio.on('disconnect', handler = handler)
            
        def set_downlink_handler(self, handler):
            self._sio.on('downlink', handler = handler)
         
        def route_message(self, payload):
            self._sio.emit('router', payload)
            
        def connect(self):
           headers = {
               "jwt_token": self._access_token,  # required
           }
        
           print("Downlink_Api - Connecting to user's {} Downlink room as user ".format(self._username))
           self._sio.connect(url = self._socket_url, transports ='websocket', headers=headers, socketio_path= '/target_is_downlink_room_as_user' ) 
    
        def disconnect(self):
           self._sio.disconnect()
            